"""Compound ZIP parser plugin for OpenXML files."""

import re
import zipfile

from xml.parsers import expat

from defusedxml import ElementTree

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.parsers import czip
from plaso.parsers.czip_plugins import interface


class OpenXMLEventData(events.EventData):
  """OXML event data.

  Attributes:
    application (str): name of application that created document.
    application_version (str): version of application that created document.
    author (str): name of author.
    creation_time (dfdatetime.DateTimeValues): creation date and time of
        the document.
    digital_signature (str): digital signature.
    edit_duration (int): total editing time.
    hyperlinks_changed (bool): True if hyperlinks have changed.
    last_printed_time (dfdatetime.DateTimeValues): date and time the document
        was last printed.
    last_saved_by (str): name of user that last saved the document.
    links_up_to_date (bool): True if the links are up to date.
    modification_time (dfdatetime.DateTimeValues): modification date and time
        of the document.
    number_of_characters (int): number of characters without spaces in
        the document.
    number_of_characters_with_spaces (int): number of characters including
        spaces in the document.
    number_of_clips (int): number of multi-media clips in the document.
    number_of_hidden_slides (int): number of hidden slides in the document.
    number_of_lines (int): number of lines in the document.
    number_of_pages (int): number of pages in the document.
    number_of_paragraphs (int): number of paragraphs in the document.
    number_of_slides (int): number of slides in the document.
    number_of_words (int): number of words in the document.
    revision_number (int): revision number.
    scale (bool): True if scaling of the thumbnail is desired or false if
        cropping is desired.
    security_flags (int): security flags.
    shared_doc (bool): True if document is shared.
    template (str): name of the template used to created the document.
  """

  DATA_TYPE = 'openxml:metadata'

  def __init__(self):
    """Initializes event data."""
    super().__init__(data_type=self.DATA_TYPE)
    self.application = None
    self.application_version = None
    self.author = None
    self.creation_time = None
    self.digital_signature = None
    self.edit_duration = None
    self.hyperlinks_changed = None
    self.last_printed_time = None
    self.last_saved_by = None
    self.links_up_to_date = None
    self.modification_time = None
    self.number_of_characters = None
    self.number_of_characters_with_spaces = None
    self.number_of_clips = None
    self.number_of_hidden_slides = None
    self.number_of_lines = None
    self.number_of_pages = None
    self.number_of_paragraphs = None
    self.number_of_slides = None
    self.number_of_words = None
    self.revision_number = None
    self.scale = None
    self.security_flags = None
    self.shared_doc = None
    self.template = None


class OpenXMLPlugin(interface.CompoundZIPPlugin):
  """Parse metadata from OXML files."""

  NAME = 'oxml'
  DATA_FORMAT = 'OpenXML (OXML) file'

  REQUIRED_PATHS = frozenset([
      '[Content_Types].xml', '_rels/.rels', 'docProps/core.xml'])

  _PROPERTY_NAMES = {
      'creator': 'author',
      'lastModifiedBy': 'last_saved_by',
      'Total_Time': 'total_edit_time',
      'Pages': 'number_of_pages',
      'CharactersWithSpaces': 'number_of_characters_with_spaces',
      'Paragraphs': 'number_of_paragraphs',
      'Characters': 'number_of_characters',
      'Lines': 'number_of_lines',
      'revision': 'revision_number',
      'Words': 'number_of_words',
      'Application': 'application',
      'Shared_Doc': 'shared'}

  def _GetPropertyValue(self, parser_mediator, properties, name):
    """Retrieves a property value.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      properties (dict[str, object]): properties.
      name (str): name of the property.

    Returns:
      str: property value.
    """
    property_value = properties.get(name)
    if isinstance(property_value, bytes):
      try:
        # TODO: get encoding form XML metadata.
        property_value = property_value.decode('utf-8')
      except UnicodeDecodeError:
        parser_mediator.ProduceExtractionWarning(
            f'unable to decode property: {name:s}')

    return property_value

  def _FormatPropertyName(self, name):
    """Formats a camel case property name as snake case.

    Args:
      name (str): property name in camel case.

    Returns:
      str: property name in snake case.
    """
    # TODO: Add Unicode support.
    fix_key = re.sub(r'(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub(r'([a-z0-9])([A-Z])', r'\1_\2', fix_key).lower()

  def _ParseBooleanValue(self, properties, name):
    """Parses a boolean property value.

    Args:
      properties (dict[str, object]): properties.
      name (str): name of the property.

    Returns:
      bool: boolean value or None not available.
    """
    string_value = properties.get(name)
    if string_value:
      if string_value == 'false':
        return False
      if string_value == 'true':
        return True

    return None

  def _ParseIntegerValue(self, properties, name):
    """Parses an integer property value.

    Args:
      properties (dict[str, object]): properties.
      name (str): name of the property.

    Returns:
      int: integer value or None not available.
    """
    string_value = properties.get(name)
    if string_value:
      try:
        return int(string_value, 10)
      except (TypeError, ValueError):
        pass

    return None

  def _ParseISO8601DateTimeString(self, parser_mediator, properties, name):
    """Parses an ISO8601 date and time string.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      properties (dict[str, object]): properties.
      name (str): name of the property.

    Returns:
      dfdatetime.TimeElementsInMicroseconds: date and time value or None if
          not available.
    """
    iso8601_string = properties.get(name)
    if not iso8601_string:
      return None

    # Date and time strings are in ISO8601 format either with 1 second
    # or 100th nano second precision. For example:
    # 2012-11-07T23:29:00Z
    # 2012-03-05T20:40:00.0000000Z
    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()

    try:
      date_time.CopyFromStringISO8601(iso8601_string)
    except ValueError as exception:
      parser_mediator.ProduceExtractionWarning((
          f'Unable to parse value: {name:s} ISO8601 string: '
          f'{iso8601_string:s} with error: {exception!s}'))
      return None

    return date_time

  def _ParsePropertiesXMLFile(self, xml_data):
    """Parses a properties XML file.

    Args:
      xml_data (bytes): data of a _rels/.rels XML file.

    Returns:
      dict[str, object]: properties.

    Raises:
      zipfile.BadZipfile: if the properties XML file cannot be read.
    """
    xml_root = ElementTree.fromstring(xml_data)

    properties = {}
    for xml_element in xml_root.iter():
      if not xml_element.text:
        continue

      # The property name is formatted as: {URL}name
      # For example: {http://purl.org/dc/terms/}modified
      _, _, name = xml_element.tag.partition('}')

      # Do not including the 'lpstr' attribute because it is very verbose.
      if name == 'lpstr':
        continue

      property_name = self._PROPERTY_NAMES.get(name)
      if not property_name:
        property_name = self._FormatPropertyName(name)

      properties[property_name] = xml_element.text

    return properties

  def _ParseRelationshipsXMLFile(self, xml_data):
    """Parses the relationships XML file (_rels/.rels).

    Args:
      xml_data (bytes): data of a _rels/.rels XML file.

    Returns:
      list[str]: property file paths. The path is relative to the root of
          the ZIP file.

    Raises:
      zipfile.BadZipfile: if the relationship XML file cannot be read.
    """
    xml_root = ElementTree.fromstring(xml_data)

    property_files = []
    for xml_element in xml_root.iter():
      type_attribute = xml_element.get('Type')
      if 'properties' in repr(type_attribute):
        target_attribute = xml_element.get('Target')
        property_files.append(target_attribute)

    return property_files

  def _ParseZIPFile(self, parser_mediator, zip_file):
    """Parses an OXML file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      zip_file (zipfile.ZipFile): the zip file containing OXML content. It is
          not be closed in this method, but will be closed by the parser logic
           in czip.py.
    """
    try:
      xml_data = zip_file.read('_rels/.rels')
      property_files = self._ParseRelationshipsXMLFile(xml_data)
    except (IndexError, KeyError, LookupError, OSError, OverflowError,
            ValueError, ElementTree.ParseError, expat.ExpatError,
            zipfile.BadZipfile) as exception:
      parser_mediator.ProduceExtractionWarning((
          f'Unable to parse relationships XML file: _rels/.rels with error: '
          f'{exception!s}'))
      return

    metadata = {}

    for path in property_files:
      try:
        xml_data = zip_file.read(path)
        properties = self._ParsePropertiesXMLFile(xml_data)
      except (IndexError, KeyError, LookupError, OSError, OverflowError,
              ValueError, ElementTree.ParseError, expat.ExpatError,
              zipfile.BadZipfile) as exception:
        parser_mediator.ProduceExtractionWarning((
            f'Unable to parse properties XML file: {path:s} with error: '
            f'{exception!s}'))
        continue

      metadata.update(properties)

    event_data = OpenXMLEventData()
    event_data.application = self._GetPropertyValue(
        parser_mediator, metadata, 'application')
    event_data.application_version = self._GetPropertyValue(
        parser_mediator, metadata, 'app_version')
    event_data.author = self._GetPropertyValue(
        parser_mediator, metadata, 'author')
    event_data.creation_time = self._ParseISO8601DateTimeString(
        parser_mediator, metadata, 'created')
    event_data.digital_signature = self._GetPropertyValue(
        parser_mediator, metadata, 'dig_sig')
    event_data.edit_duration = self._ParseIntegerValue(metadata, 'total_time')
    event_data.hyperlinks_changed = self._ParseBooleanValue(
        metadata, 'hyperlinks_changed')
    # event_data.i4 = self._ParseIntegerValue(
    #     parser_mediator, metadata, 'i4')
    event_data.last_printed_time = self._ParseISO8601DateTimeString(
        parser_mediator, metadata, 'last_printed')
    event_data.last_saved_by = self._GetPropertyValue(
        parser_mediator, metadata, 'last_saved_by')
    event_data.links_up_to_date = self._ParseBooleanValue(
        metadata, 'links_up_to_date')
    event_data.modification_time = self._ParseISO8601DateTimeString(
        parser_mediator, metadata, 'modified')
    event_data.number_of_characters = self._ParseIntegerValue(
        metadata, 'number_of_characters')
    event_data.number_of_characters_with_spaces = self._ParseIntegerValue(
        metadata, 'number_of_characters_with_spaces')
    event_data.number_of_clips = self._ParseIntegerValue(metadata, 'mm_clips')
    event_data.number_of_hidden_slides = self._ParseIntegerValue(
        metadata, 'hidden_slides')
    event_data.number_of_lines = self._ParseIntegerValue(
        metadata, 'number_of_lines')
    event_data.number_of_pages = self._ParseIntegerValue(
        metadata, 'number_of_pages')
    event_data.number_of_paragraphs = self._ParseIntegerValue(
        metadata, 'number_of_paragraphs')
    event_data.number_of_slides = self._ParseIntegerValue(metadata, 'slides')
    event_data.number_of_words = self._ParseIntegerValue(
        metadata, 'number_of_words')
    event_data.revision_number = self._ParseIntegerValue(
        metadata, 'revision_number')
    event_data.scale = self._ParseBooleanValue(metadata, 'scale_crop')
    event_data.security_flags = self._ParseIntegerValue(
        metadata, 'doc_security')
    event_data.shared_doc = self._GetPropertyValue(
        parser_mediator, metadata, 'shared_doc')
    event_data.template = self._GetPropertyValue(
        parser_mediator, metadata, 'template')

    parser_mediator.ProduceEventData(event_data)


czip.CompoundZIPParser.RegisterPlugin(OpenXMLPlugin)



"""SQLite parser plugin for Android application usage database files."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class AndroidAppUsage(events.EventData):
  """Android application usage event data.

  Attributes:
    package_name (str): name of the launched package.
    start_time (dfdatetime.DateTimeValues): date and time when the application
        was launched.
  """

  DATA_TYPE = 'android:sqlite:app_usage'

  def __init__(self):
    """Initializes event data."""
    super().__init__(data_type=self.DATA_TYPE)
    self.package_name = None
    self.start_time = None


class AndroidSQLiteAppUsage(interface.SQLitePlugin):
  """SQLite parser plugin for Android application usage database files."""

  NAME = 'android_app_usage'
  DATA_FORMAT = 'Android application usage SQLite database (app_usage) file'

  REQUIRED_STRUCTURE = {
      'events': frozenset(['_id', 'timestamp', 'package_id']),
      'packages': frozenset(['_id', 'package_name'])}

  QUERIES = [
      ('SELECT events.timestamp, packages.package_name FROM events JOIN '
       'packages ON packages._id = events.package_id', 'ParseAppUsageRow')]

  SCHEMAS = [{
      'events': (
          'CREATE TABLE "events" (_id INTEGER PRIMARY KEY,timestamp INTEGER '
          'NOT NULL,type INTEGER NOT NULL,package_id INTEGER NOT NULL '
          'REFERENCES packages(_id) ON UPDATE CASCADE ON DELETE CASCADE, '
          'instance_id INTEGER DEFAULT NULL, task_root_package_id INTEGER '
          'DEFAULT NULL REFERENCES packages(_id) ON UPDATE CASCADE ON DELETE '
          'CASCADE)'),
      'packages': (
          'CREATE TABLE packages (_id INTEGER PRIMARY KEY,package_name TEXT, '
          'UNIQUE(package_name) ON CONFLICT ABORT)')}]

  def ParseAppUsageRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses an event record row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    timestamp = self._GetRowValue(query_hash, row, 'timestamp')

    event_data = AndroidAppUsage()
    event_data.package_name = self._GetRowValue(query_hash, row, 'package_name')
    event_data.start_time = dfdatetime_posix_time.PosixTimeInMilliseconds(
        timestamp=timestamp)

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(AndroidSQLiteAppUsage)



"""SQLite parser plugin for Android Viber call history database files."""

from dfdatetime import java_time as dfdatetime_java_time

from plaso.containers import events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class AndroidViberCallEventData(events.EventData):
  """Android Viber Call event data.

  Attributes:
    call_type (int): type of call, such as Incoming or Outgoing.
    duration (int): number of seconds the call lasted.
    end_time (dfdatetime.DateTimeValues): date and time the call was stopped.
    phone_number (str): phone number.
    start_time (dfdatetime.DateTimeValues): date and time the call was started.
    viber_call_type (int): type of call in Viber, such as Audio Call, or Video
        Call.
  """

  DATA_TYPE = 'android:viber:call'

  def __init__(self):
    """Initializes event data."""
    super().__init__(data_type=self.DATA_TYPE)
    self.call_type = None
    self.duration = None
    self.end_time = None
    self.phone_number = None
    self.start_time = None
    self.viber_call_type = None


class AndroidViberCallPlugin(interface.SQLitePlugin):
  """SQLite parser plugin for Android Viber call history database files.

  The Android Viber call history database file is typically stored in:
  viber_data
  """

  NAME = 'android_viber_call'
  DATA_FORMAT = 'Android Viber call history SQLite database (viber_data) file'

  REQUIRED_STRUCTURE = {
      'calls': frozenset([
          '_id', 'date', 'number', 'duration', 'type', 'viber_call_type'])}

  QUERIES = [
      ('SELECT _id AS id, date, number, duration, type, viber_call_type '
       'FROM calls', 'ParseViberCallsRow')]

  SCHEMAS = [{
    'blockednumbers': (
        'CREATE TABLE blockednumbers (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
        'canonized_number TEXT NOT NULL, blocked_date LONG DEFAULT 0, '
        'block_reason INTEGER DEFAULT 0, status INTEGER DEFAULT 0, '
        'UNIQUE (canonized_number) ON CONFLICT REPLACE )'),
    'calls': (
        'CREATE TABLE calls (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
        'call_id LONG NOT NULL, aggregate_hash LONG NOT NULL, '
        'number TEXT NOT NULL, canonized_number TEXT NOT NULL, '
        'viber_call BOOLEAN DEFAULT TRUE, viber_call_type INTEGER DEFAULT 1, '
        'date LONG NOT NULL, duration LONG NOT NULL, type INT NOT NULL, '
        'end_reason INT DEFAULT 0, start_reason INT DEFAULT 0, '
        'token LONG DEFAULT 0, looked BOOLEAN DEFAULT TRUE, '
        'conference_info TEXT, group_id LONG DEFAULT 0, '
        'flags INTEGER DEFAULT 0)'),
    'phonebookcontact': (
        'CREATE TABLE phonebookcontact (_id INTEGER PRIMARY KEY NOT NULL, '
        'native_id INTEGER DEFAULT 0, display_name TEXT, phonetic_name TEXT, '
        'phone_label TEXT, low_display_name TEXT, starred BOOLEAN, '
        'viber BOOLEAN, contact_lookup_key TEXT, contact_hash LONG, '
        'version INTEGER DEFAULT 0, has_number BOOLEAN, has_name BOOLEAN, '
        'native_photo_id LONG DEFAULT 0, recently_joined_date LONG DEFAULT 0, '
        'joined_date LONG DEFAULT 0, numbers_name TEXT DEFAULT '', '
        'deleted BOOLEAN, flags INTEGER DEFAULT 0, '
        'UNIQUE (_id) ON CONFLICT REPLACE)'),
    'sqlite_sequence': (
        'CREATE TABLE sqlite_sequence(name,seq)'),
    'sync_data': (
        'CREATE TABLE sync_data (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
        'canonized_phone_number TEXT NOT NULL, display_name TEXT DEFAULT '', '
        'phonetic_name TEXT DEFAULT '',operation INTEGER  DEFAULT 0)'),
    'vibernumbers': (
        'CREATE TABLE vibernumbers (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
        'canonized_number TEXT NOT NULL, photo TEXT DEFAULT '', '
        'viber_name TEXT, clear BOOLEAN, member_id TEXT NOT NULL, '
        'encrypted_member_id TEXT NOT NULL, viber_id TEXT, '
        'date_of_birth TEXT)'),
    'viberpay_data': (
        'CREATE TABLE viberpay_data (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
        'encrypted_member_id TEXT NULL, canonized_phone_number TEXT NULL, '
        'phone_number TEXT NULL, country_code TEXT NULL, '
        'is_country_supported BOOLEAN NOT NULL, '
        'default_currency_code TEXT NULL, is_viberpay_user BOOLEAN NOT NULL, '
        'last_sync_date INTEGER NOT NULL)'),
    'walletlist': (
        'CREATE TABLE walletlist (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
        'country_codes INTEGER NOT NULL)'),
    'walletnumbers': (
        'CREATE TABLE walletnumbers (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
        'canonized_number TEXT NOT NULL, wallet_wu_status INTEGER DEFAULT 0, '
        'UNIQUE (canonized_number) ON CONFLICT REPLACE )'),
    }]

  def ParseViberCallsRow(self, parser_mediator, query, row, **unused_kwargs):
    """Parses a Viber Call record row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    duration = self._GetRowValue(query_hash, row, 'duration')
    timestamp = self._GetRowValue(query_hash, row, 'date')

    event_data = AndroidViberCallEventData()
    event_data.call_type = self._GetRowValue(query_hash, row, 'type')
    event_data.duration = duration
    event_data.phone_number = self._GetRowValue(query_hash, row, 'number')
    event_data.viber_call_type = self._GetRowValue(
        query_hash, row, 'viber_call_type')
    event_data.start_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)

    if duration:
      # The duration is in seconds and the date value in milliseconds.
      timestamp += duration * definitions.MILLISECONDS_PER_SECOND

      event_data.end_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(AndroidViberCallPlugin)



"""JSON-L parser plugin for AWS CloudTrail log files."""

import json

from json import decoder as json_decoder

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.parsers import jsonl_parser
from plaso.parsers.jsonl_plugins import interface


class AWSCloudTrailEventData(events.EventData):
  """AWS CloudTrail log event data.

  Attributes:
    access_key (str): access key identifier.
    account_identifier (str): AWS account identifier.
    cloud_trail_event (str): CloudTrail event.
    event_name (str): event name.
    event_source (str): AWS service.
    recorded_time (dfdatetime.DateTimeValues): date and time the log entry
        was recorded.
    resources (str): resources.
    source_ip (str): source IP address.
    user_identity_arn (str): AWS ARN of the user.
    user_name (str): name of the AWS user.
  """

  DATA_TYPE = 'aws:cloudtrail:entry'

  def __init__(self):
    """Initializes event data."""
    super().__init__(data_type=self.DATA_TYPE)
    self.access_key = None
    self.account_identifier = None
    self.cloud_trail_event = None
    self.event_name = None
    self.event_source = None
    self.recorded_time = None
    self.resources = None
    self.source_ip = None
    self.user_identity_arn = None
    self.user_name = None


class AWSCloudTrailLogJSONLPlugin(interface.JSONLPlugin):
  """JSON-L parser plugin for AWS CloudTrail log files."""

  NAME = 'aws_cloudtrail_log'
  DATA_FORMAT = 'AWS CloudTrail Log'

  def _ParseRecord(self, parser_mediator, json_dict):
    """Parses an AWS CloudTrail log record.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      json_dict (dict): JSON dictionary of the log record.
    """
    date_time = None

    event_time = self._GetJSONValue(json_dict, 'EventTime')
    if event_time:
      try:
        date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()
        date_time.CopyFromDateTimeString(event_time)
      except ValueError as exception:
        parser_mediator.ProduceExtractionWarning(
            f'Unable to parse EventTime value: {event_time:s} with error: '
            f'{exception!s}')
        date_time = None

    resource_list = self._GetJSONValue(
        json_dict, 'Resources', default_value=[])
    # Flatten multiple resources into a string of resource names.
    resources = ', '.join(
        [resource['ResourceName'] for resource in resource_list])

    cloud_trail_event = self._GetJSONValue(
        json_dict, 'CloudTrailEvent', default_value='{}')
    try:
      cloud_trail_event_json = json.loads(cloud_trail_event)
    except json_decoder.JSONDecodeError as exception:
      parser_mediator.ProduceExtractionWarning(
          f'Unable to decode CloudTrail event with error: {exception!s}')
      cloud_trail_event_json = {}

    user_identity_json = self._GetJSONValue(
        cloud_trail_event_json, 'userIdentity')

    event_data = AWSCloudTrailEventData()
    event_data.access_key = self._GetJSONValue(json_dict, 'AccessKeyId')
    event_data.account_identifier = self._GetJSONValue(
        cloud_trail_event_json, 'accountId')
    event_data.cloud_trail_event = cloud_trail_event
    event_data.event_name = self._GetJSONValue(json_dict, 'EventName')
    event_data.event_source = self._GetJSONValue(json_dict, 'EventSource')
    event_data.recorded_time = date_time
    event_data.resources = resources or None
    event_data.source_ip = self._GetJSONValue(
        cloud_trail_event_json, 'sourceIPAddress')
    event_data.user_identity_arn = self._GetJSONValue(user_identity_json, 'arn')
    event_data.user_name = self._GetJSONValue(json_dict, 'Username')

    parser_mediator.ProduceEventData(event_data)

  def CheckRequiredFormat(self, json_dict):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      json_dict (dict): JSON dictionary of the log record.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    cloud_trail_event = self._GetJSONValue(json_dict, 'CloudTrailEvent')
    event_time = self._GetJSONValue(json_dict, 'EventTime')

    if None in (cloud_trail_event, event_time):
      return False

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()

    try:
      date_time.CopyFromDateTimeString(event_time)
    except ValueError:
      return False

    return True


jsonl_parser.JSONLParser.RegisterPlugin(AWSCloudTrailLogJSONLPlugin)
